@@ -49,25 +49,9 @@ def load_cp(instream, where=None):
 
 
 @savers.register(linked=load_cp)
-def save_cp(fonts, outstream, where=None, codepage_prefix:str='cp'):
-    """
-    Save character-cell fonts to Linux Keyboard Codepage (.CP) file.
-
-    codepage_prefix: prefix to use to find numbered codepage in encodings. Default: 'cp'.
-    """
-    fonts = _make_fit(fonts, codepage_prefix)
-    cpdata, _ = _convert_to_cp(fonts)
-    if len(cpdata) > 1:
-        raise FileFormatError(
-            'All fonts in a single .cp file must have the same encoding.'
-        )
-    _write_cp(outstream, cpdata[0])
-
-
-@savers.register(linked=load_cpi)
-def save_cpi(
+def save_cp(
         fonts, outstream, where=None,
-        cpi_version:str=_ID_MS, codepage_prefix:str='cp'
+        version:str=_ID_MS, codepage_prefix:str='cp'
     ):
     """
     Save character-cell fonts to Linux Keyboard Codepage (.CP) file.
@@ -75,7 +59,30 @@ def save_cpi(
     version: CPI format version. One of 'DRFONT', 'FONT.NT', or 'FONT' (default)
     codepage_prefix: prefix to use to find numbered codepage in encodings. Default: 'cp'.
     """
-    format = cpi_version[:7].upper().ljust(7)
+    format = version[:7].upper().ljust(7)
+    if isinstance(format, str):
+        format = format.encode('ascii', 'replace')
+    fonts = _make_fit(fonts, codepage_prefix)
+    cpdata, _ = _convert_to_cp(fonts)
+    if len(cpdata) > 1:
+        raise FileFormatError(
+            'All fonts in a single .cp file must have the same encoding.'
+        )
+    _write_cp(outstream, cpdata[0], format=format)
+
+
+@savers.register(linked=load_cpi)
+def save_cpi(
+        fonts, outstream, where=None,
+        version:str=_ID_MS, codepage_prefix:str='cp'
+    ):
+    """
+    Save character-cell fonts to Linux Keyboard Codepage (.CP) file.
+
+    version: CPI format version. One of 'DRFONT', 'FONT.NT', or 'FONT' (default)
+    codepage_prefix: prefix to use to find numbered codepage in encodings. Default: 'cp'.
+    """
+    format = version[:7].upper().ljust(7)
     if isinstance(format, str):
         format = format.encode('ascii', 'replace')
     if format in (_ID_MS, _ID_NT):
@@ -448,7 +455,7 @@ def _convert_to_cp(fonts):
             device_type=_DT_SCREEN,
             device_name=device,
             codepage=cp_number,
-            cpih_offset=0,
+            #cpih_offset=0,
         )
         cp_output.cpih = _CODEPAGE_INFO_HEADER(
             #version=_CP_DRFONT if format==_ID_DR else _CP_FONT,
